/*-------------------------------------------------------------
 *  Copyright (c) 2010, MSS ComBlock. All rights reserved.
 *
 * 	Filename: lpc1759_rom.ld
 *  Created on: Jun 24, 2010
 *  Last modified:
 *  Authors:
 *  	Alain Zarembowitch / MSS ComBlock
 *  Version: 1.0
 *  Inheritance: N/A
 *  Description: loader for flash-based program.
 *  512KB flash (LPC1759) mostly for code, although flash sector 1 (4KB starting at address 0x1000) is used to save ComBlock configuration data
 *  tutorial: http://ftp.gnu.org/old-gnu/Manuals/ld-2.9.1/html_chapter/ld_3.html
 *
 *  Dependencies/Pre-requisite: N/A
 -------------------------------------------------------------*/

/* Stack Sizes */

   _STACKSIZE = 4096;
   _HEAPSIZE = 4096;	/* enough room for a buf[2048] local array and some, as declared in some functions */

/* Memory Definitions */

MEMORY
{
  rom (rx)  : ORIGIN = 0x00000000, LENGTH = 512K
  ram (rwx) : ORIGIN = 0x10000000, LENGTH =  32K
  
  ram1(rwx) : ORIGIN = 0x2007C000, LENGTH = 16k
  ram2(rwx) : ORIGIN = 0x20080000, LENGTH = 16k
}

/* Section Definitions */

SECTIONS
{
   /* interrupt vector table must be located at address 0 */
   .vectors : {
      KEEP(*(.vectors))
   } >rom =0
   . = ALIGN(4);

   /* code read protection at address 0x1FC */
   .ARM.__at_0x1FC 0x1FC : {
	KEEP(*(.ARM.__at_0x1FC))
   } >rom
   . = ALIGN(4);


   /* place code starting at flash sector 2, thus skipping sector 1 which is used for saving 4KB of ComBlock configuration data */
   .text 0x2000 :
   {
      CREATE_OBJECT_SYMBOLS
      *(.text .text.*)
      *(.gnu.linkonce.t.*)
      *(.glue_7t) *(.glue_7) *(.vfp11_veneer)
      KEEP(*(.fini))
      *(.gcc_except_table)
   } >rom
   . = ALIGN(4);

   /* .ctors .dtors are used for c++ constructors/destructors */
   
   .ctors :
   {
      PROVIDE(__ctors_start__ = .);
      KEEP(*(SORT(.ctors.*)))
      KEEP(*(.ctors))
      PROVIDE(__ctors_end__ = .);
   } >rom

   .dtors :
   {
      PROVIDE(__dtors_start__ = .); 
      KEEP(*(SORT(.dtors.*)))
      KEEP(*(.dtors))
      PROVIDE(__dtors_end__ = .);
   } >rom
   
   /* .rodata section which is used for read-only data (constants) */

   .rodata :
   {
      *(.rodata .rodata.*)
      *(.gnu.linkonce.r.*)
   } >rom
   . = ALIGN(4);

   .init_array :
   {
      *(.init)
        *(.fini)
      PROVIDE_HIDDEN (__preinit_array_start = .);
      KEEP (*(.preinit_array))
      PROVIDE_HIDDEN (__preinit_array_end = .);
      PROVIDE_HIDDEN (__init_array_start = .);
      KEEP (*(SORT(.init_array.*)))
      KEEP (*(.init_array))
      PROVIDE_HIDDEN (__init_array_end = .);
      PROVIDE_HIDDEN (__fini_array_start = .);
      KEEP (*(.fini_array))
      KEEP (*(SORT(.fini_array.*)))
      PROVIDE_HIDDEN (__fini_array_end = .);
   } >rom

   . = ALIGN(4);

   /* .ARM.exidx is sorted, so has to go in its own output section.  */
   __exidx_start = .;
   .ARM.exidx :
   {
      *(.ARM.exidx* .gnu.linkonce.armexidx.*)
   } >rom
   __exidx_end = .;

   _etext = .;
   PROVIDE (etext = .);

   /* .data section which is used for initialized data */

   .data :
   {
      __data_start = .;
      *(.data .data.*)
      *(.gnu.linkonce.d.*)
      SORT(CONSTRUCTORS)
      . = ALIGN(4);
      *(.fastrun .fastrun.*)
   } >ram AT>rom
   . = ALIGN(4);
   
   _edata = .;
   PROVIDE (edata = .);

   /* .bss section which is used for uninitialized data */

   .bss :
   {
      __bss_start = .;
      __bss_start__ = .;
      *(.bss .bss.*)
      *(.gnu.linkonce.b.*)
      *(COMMON)
      . = ALIGN(4);
   } >ram AT>rom
   __bss_end__ = .;
   
   _end = .;
   PROVIDE(end = .);

	/* This used for USB RAM section */
	.usb_ram (NOLOAD):
	{
		*.o (USB_RAM)
	} >ram2


   /* .heap section which is used for memory allocation */
   
   .heap (NOLOAD) :
   {
      __heap_start__ = .;
      *(.heap)
      . = MAX(__heap_start__ + _HEAPSIZE , .);
   } >ram
   __heap_end__ = __heap_start__ + SIZEOF(.heap);
   
   /* .stack section - user mode stack */
   
   .stack (__heap_end__ + 3) / 4 * 4 (NOLOAD) :
   {
      . = ALIGN(8);
      __stack_start__ = .;
      *(.stack)
      . = ALIGN(8);
      . = MAX(__stack_start__ + _STACKSIZE , .);
   } >ram
   __stack_end__ = __stack_start__ + SIZEOF(.stack);

   /* Stabs debugging sections.  */
   .stab          0 : { *(.stab) }
   .stabstr       0 : { *(.stabstr) }
   .stab.excl     0 : { *(.stab.excl) }
   .stab.exclstr  0 : { *(.stab.exclstr) }
   .stab.index    0 : { *(.stab.index) }
   .stab.indexstr 0 : { *(.stab.indexstr) }
   .comment       0 : { *(.comment) }
   /* DWARF debug sections.
      Symbols in the DWARF debugging sections are relative to the beginning
      of the section so we begin them at 0.  */
   /* DWARF 1 */
   .debug          0 : { *(.debug) }
   .line           0 : { *(.line) }
   /* GNU DWARF 1 extensions */
   .debug_srcinfo  0 : { *(.debug_srcinfo) }
   .debug_sfnames  0 : { *(.debug_sfnames) }
   /* DWARF 1.1 and DWARF 2 */
   .debug_aranges  0 : { *(.debug_aranges) }
   .debug_pubnames 0 : { *(.debug_pubnames) }
   /* DWARF 2 */
   .debug_info     0 : { *(.debug_info .gnu.linkonce.wi.*) }
   .debug_abbrev   0 : { *(.debug_abbrev) }
   .debug_line     0 : { *(.debug_line) }
   .debug_frame    0 : { *(.debug_frame) }
   .debug_str      0 : { *(.debug_str) }
   .debug_loc      0 : { *(.debug_loc) }
   .debug_macinfo  0 : { *(.debug_macinfo) }
   /* SGI/MIPS DWARF 2 extensions */
   .debug_weaknames 0 : { *(.debug_weaknames) }
   .debug_funcnames 0 : { *(.debug_funcnames) }
   .debug_typenames 0 : { *(.debug_typenames) }
   .debug_varnames  0 : { *(.debug_varnames) }  
}
